 /*****************************************************************************
  *                                                                           
  *  Copyright (c) 2008 Johannes Hermen www.der-frickler.net                   
  *                                                                           
  *                                                                           
  *  This library is free software; you can redistribute it and/or modify it  
  *  under the terms of the GNU Lesser General Public License as published    
  *  by the Free Software Foundation; either version 2 of the License, or     
  *  (at your option) any later version.                                      
  *                                                                           
  *  This software is distributed in the hope that it will be useful, but     
  *  WITHOUT ANY WARRANTY; without even the implied warranty of               
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        
  *  Lesser General Public License for more details.                          
  *                                                                           
  *  You should have received a copy of the GNU Lesser General Public         
  *  License along with this library; if not, write to the Free Software      
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA  
  *                                                                           
  *****************************************************************************
  *                                                                           
  *  Signal Splitter TINY13                                                   
  *  Splits an RC-SUM Signal into the single channels and outputs the         
  *  single servo signals on the output pins. Can be used to connect a        
  *  brushless ESC (and normal Servo) to an original Micro-T electronic.      
  *                                                                           
  *  PINOUT: Sum Signal IN: PIN 5 (PB0)                                       
  *          CHANNEL 1 OUT: PIN 2 (PB3)                                       
  *          CHANNEL 2 OUT: PIN 3 (PB4)                                       
  *                                                                           
  *****************************************************************************/

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>


unsigned int value;
unsigned int channel;


// on timer0 overflow
ISR (TIM0_OVF_vect)	
{
}

// on PB0 pinchange
ISR (PCINT0_vect)
{
  if ((PINB & 1)==0) 	// check 1st bit of PORTB (pin 5) got LOW
  {

    TCNT0 = 0; 			// reset timer 
    TCCR0B = 5;			// start timer (prescaler :1024)
    value = 0;			// measured time = 0
	
    if (channel > 0 ) {     // if signal is valid ( after first startseq.)
        // set the output pins for corresponding channel
        if (channel == 1) {  
            // set PB3 (pin 2) = Steering Signal
			PORTB = 8;
		} else if (channel == 2) {  
            // set PB4 (pin 3) = Motor Signal
			PORTB = 16;
		} else  {
			PORTB = 0;
        }
        channel +=1; // inc. channel
    }   
  }
  else {	// check 1st bit of PORTB (pin 5) got HIGH
	TCCR0B = 0; 		// stop timer0
    value = TCNT0; 		// get timer0 value
	PORTB = 0;			// reset portb to 
	if (value > 10) {	// if low value longer than 3ms -> startpulse found
		channel = 1;	// reset channels
	} 
	
  }
}

int main (void)

{

  	DDRB = 254;	// portb PB0 as input, rest as output
	PORTB = 0; 	// portb to 0	

  	TCCR0B = 0; // stop timer0
  	TIMSK0 = 2; // timer0 Overflow Interrupt Enable
  	TCNT0 = 0; // timer startval = 0 


	GIMSK = 32;	// enable Ext Interrupt on PinChange
  	PCMSK = 1; 	// set Ext Interrupt on PB0 (pin 5)

	channel = -1;	// output channel

  	sei(); 	// Interrupts on.

	while(1) // do nothing but wait
	{
		_delay_ms(1000);
	}

}
